<#
 Pre-Requisites:

 1) Run PowerShell as an Administrator.
 2) Ensure the "PnP.PowerShell" module is installed. If not, the script will install it automatically.
 3) Set the execution policy for external scripts: `Set-ExecutionPolicy -ExecutionPolicy RemoteSigned -Scope Process -Force`.
 4) To enable custom scripts, you need either the SharePoint Online Management Shell or the "Microsoft.Online.SharePoint.PowerShell" module (not both). This script requires the "Microsoft.Online.SharePoint.PowerShell" module.
#>

$cwd = $pwd.Path;
Write-Host $cwd
$newline = [Environment]::NewLine;

$webPartPageName = "MuhimbiFeatures.aspx"

# The following have to be exactly the same as they are in the MuhimbiPdfConverterForSharePointOnlineWebPart.manifest.json
$muhimbiWebpartName = "Muhimbi PDF Converter for SharePoint Online"

# The following must be the same as in the package-solution.json
$muhimbiPackageName = "Muhimbi PDF converter for SharePoint Online SPFx";
$muhimbiSolutionID = "5316e4b6-35a7-4d55-ac35-b3974c165f2c";

Function Show-Menu
{

    Write-Host ""
    #Clear-Host
    Write-Host "================ Muhimbi PDF Converter online for SharePoint Online (web part) ================$newline"
    Write-Host "1: Press '1' to perform a fresh installation of the Muhimbi PDF Converter online app."
    Write-Host "2: Press '2' to create a web part page in a site."
	Write-Host "3: Press '3' to create a web part page on all sites."
    Write-Host "4: Press '4' to remove a web part page from a site."
	Write-Host "5: Press '5' to remove web part page from all sites."
    Write-Host "6: Press '6' to remove the Muhimbi PDF Converter online SPFx app from the app catalog."
    Write-Host "7: Press '7' to enable external user group in tenant."
    Write-Host "8: Press '8' to elevate the Muhimbi PDF Converter online App on a specific site collection."
	Write-Host "9: Press '9' to elevate the Muhimbi PDF Converter online App on all site collections."
	Write-Host "10: Press '10' to revoke the elevation for the Muhimbi PDF Converter online App from a specific site collection."
	Write-Host "11: Press '11' to revoke the elevation for the Muhimbi PDF Converter online App from all site collections."
	Write-Host "12: Press '12' to enable custom script in a specific site collection."
	Write-Host "13: Press '13' to enable custom script on all site collection."
    Write-Host "14: Press '14' to remove old Muhimbi PDF Converter online add in from all site collections."
    Write-Host "Q: Press 'Q' to quit."
    Write-Host ""
}

Function Run()
{
    try
    {
        $logFile = "$cwd\log.txt"
        Start-Transcript -Append $logFile

        do
        {
            Show-Menu
            $selection = Read-Host "Please make a selection"
            switch ($selection)
            {
                '1' { InstallApp; }
                '2' { CreateWebPage; }
				'3' { CreateWebPageAllSite; }
                '4' { RemoveWebPage; }
				'5' { RemoveWebPageFromAllSites; }
                '6' { UninstallApp; }
                '7' { TurnOnExternalSharing; }
                '8' { GiveFullControlToSiteCollection; }
				'9' { GiveFullControlToAllSites; }
				'10' { RevokeFullControlFromSiteCollection; }
				'11' { RevokeFullControlFromAllSites; }
				'12' { EnableCustomScript; }
				'13' { EnableCustomScriptOnAllSites; }
                '14' { UninstallOldMuhimbiAddInAllSites; }
            }

            Write-Host "$newline---------------------------------------------------------------------------------$newline"
        }
        until ($selection -eq 'q')
    }
    finally
    {
        Stop-Transcript
    }
}

Function InstallApp()
{
    try
    {
        $selection = "0";
        $siteUrl = "";
		$tenantUrl = "";
        Write-Host "";

        do
        {
            Write-Host "Where do you want to upload the app?";
            Write-Host "1: Press '1' to upload the app in the tenant app catalog."
            Write-Host "2: Press '2' to upload the app in a site app catalog."
            $selection = Read-Host "Please make a selection"
            Write-Host "";
        }
        until ($selection -eq '1' -Or $selection -eq '2')

        if ($selection -eq '1')
        {
            $scope = "tenant";
        }
        else
        {
            $scope = "site";
        }

        $siteUrl = GetStringValueFromUser -message "SharePoint $scope URL"
        Connect-PnPOnline -Url $siteUrl -Interactive

        $sppkgFileName = "muhimbi-pdf-converter-spfx-app.sppkg";
        $sppkgFilePath = "$cwd\$sppkgFileName";

        if (![System.IO.File]::Exists($sppkgFilePath))
        {
            Write-Host -ForegroundColor Red "Could not find the file '$sppkgFilePath'"
            return;
        }

        $canCreateWebPartPage = $true;
		$createWebPartPageOnAllSites = $false;

        if ($selection -eq '1')
        {			
            $appCatalogUrl = Get-PnPTenantAppCatalogUrl

            if ([String]::IsNullOrWhiteSpace($appCatalogUrl))
            {
                Write-Host -ForegroundColor Red "The SharePoint tenant does not currently have an App catalog. Please create one and try again.";
                return;
            }

            Write-Host "App catalog URL: $appCatalogUrl";
            Write-Host 'Uploading app to tenant app catalog' -ForegroundColor Cyan;

            $spfx = Add-PnPApp -Path $sppkgFilePath -Scope Tenant -Overwrite
            Write-Host "Upload successful" -ForegroundColor Green;

            if (!$spfx.Deployed)
            {
                $decision = Confirm -title "Confirm" -question "Do you want to deploy the app to all sites in the tenant?";

                Write-Host "";

                if ($decision -eq 0)
                {
                    Write-Host "Deploying app and making it available to all sites" -ForegroundColor Cyan;
                    Publish-PnPApp -Identity $spfx.Id -SkipFeatureDeployment -Scope Tenant
                    Write-Host 'Deployment successful' -ForegroundColor Green;
					$createWebPartPageOnAllSites = $true;
					$tenantUrl = $siteUrl
                }
                else
                {
                    Write-Host 'Deploying app' -ForegroundColor Cyan;
                    Publish-PnPApp -Identity $spfx.Id -Scope Tenant

                    $decision = Confirm -title "Confirm" -question "The app has been deployed but is not installed in any sites. Do you want to install the app in a site?";

                    if ($decision -eq 0)
                    {
                        $decision = Confirm -title "Confirm" -question "Press 'Y' to install the app in the site: '$siteUrl'. Press 'N' to choose a different site."

                        if ($decision -ne 0)
                        {
                            $siteUrl = GetStringValueFromUser -message 'Please enter the URL of the site where you want to install the app';
                            Connect-PnPOnline -Url $siteUrl -Interactive
                        }

                        Install-PnPApp -Identity $spfx.Id
                        Write-Host "App successfully installed in '$siteurl' $newline" -ForegroundColor Green;
                    }
                    else
                    {
                        $canCreateWebPartPage = $false;
                    }
                }
            }
        }
        else
        {
            $appCatalog = Get-PnPSiteCollectionAppCatalog -CurrentSite

            if ($null -eq $appCatalog)
            {
                $decision = Confirm -title "Confirm" -question "The SharePoint site does not currently have an app catalog. Do you want to create one?";

                if ($decision -eq 0)
                {
                    Add-PnPSiteCollectionAppCatalog -Site $siteUrl
                    Write-Host "App catalog successfully created '$siteurl' $newline" -ForegroundColor Green;
                }
                else
                {
                    Write-Host "Cannot upload app because there are no site collection app catalog. Aborting..." -ForegroundColor Red
                    return;
                }
            }

            $appCatalog = Get-PnPSiteCollectionAppCatalog -CurrentSite
            if ($null -eq $appCatalog)
            {
                Write-Host "Cannot upload app because there are no app catalog in the site. Aborting..." -ForegroundColor Red
                return;
            }

            $appCatalogUrl = $appCatalog.AbsoluteUrl;
            Write-Host "App catalog URL: $appCatalogUrl";
            Write-Host 'Uploading app to site app catalog' -ForegroundColor Cyan;

            $spfx = Add-PnPApp -Path $sppkgFilePath -Scope Site -Overwrite
            Write-Host "Upload successful $newline" -ForegroundColor Green;

            if (!$spfx.Deployed)
            {
                Write-Host 'Deploying app' -ForegroundColor Cyan;
                Publish-PnPApp -Identity $spfx.Id -Scope Site

                $decision = Confirm -title "Confirm" -question "The app has been deployed but is not installed. Do you want to install the app in this site?";

                if ($decision -eq '0')
                {
                    Install-PnPApp -Identity $spfx.Id -Scope Site
                    Write-Host "App successfully installed in '$siteurl' $newline" -ForegroundColor Green;
                }
                else
                {
                    $canCreateWebPartPage = $false;
                }

            }
        }

        TryToGrantPermissions -siteUrl $siteUrl

		if($createWebPartPageOnAllSites)
		{
			$decision = Confirm -title "Confirm" -question "Do you want to create a page and add the Muhimbi Features web part to all site collections?";

            if ($decision -eq 0)
            {
				CreateWebPageAllSite -tenantURL $tenantURL
			}
		}
        if ($canCreateWebPartPage -and -not $createWebPartPageOnAllSites)
        {
            $decision = Confirm -title "Confirm" -question "Do you want to create a page and add the Muhimbi Features web part?";

            if ($decision -eq 0)
            {
                if ($selection -eq '1')
                {
                    $decision = Confirm -title "Confirm" -question "Press 'Y' to create the page in the site: '$siteUrl'. Press 'N' to choose a different site."

                    if ($decision -ne 0)
                    {
                        $siteUrl = GetStringValueFromUser -message 'Please enter the URL of the site where you want to create the page';
                        Connect-PnPOnline -Url $siteUrl -Interactive
                    }
                }

                CreateWebPage -siteURL $siteUrl
            }
        }

        Write-Host $newline
        Write-Host "Installation completed successfully" -ForegroundColor Green
        Write-Host "You may want to go to 'https://tenantname-admin.sharepoint.com/_layouts/15/online/AdminHome.aspx#/webApiPermissionManagement' to approve the permission request manually" -ForegroundColor Green
    }
    catch
    {
        Write-Host "Failed to install Muhimbi PDF Converter online." -ForegroundColor Red
        Write-Host $_ -ForegroundColor Red
    }
}

Function TryToGrantPermissions([string] $siteUrl)
{
	$grantedAllPermissions = $true;
    try
    {
		$permissionRequests = Get-PnPTenantServicePrincipalPermissionRequests
		
		Write-Host "";
        foreach ($req in $permissionRequests)
        {
            if ($req.PackageName -eq "$muhimbiPackageName")
            {
				Write-Host ($req | Format-Table | Out-String)
                Write-Host "Granting permissions $($req.Name) $($req.Scope)"

                Approve-PnPTenantServicePrincipalPermissionRequest -RequestId $req.Id -Force -ErrorAction SilentlyContinue -ErrorVariable approveRequestError -OutVariable approveRequestResult | Out-Null
                # Grant-PnPTenantServicePrincipalPermission -Scope $req.Scope -Resource $req.Resource

                if (![String]::IsNullOrWhiteSpace($approveRequestError) -And -Not($approveRequestError -imatch "already exists"))
                {
                    Write-Host "approveRequestError: $approveRequestError"
                    $grantedAllPermissions = $false;
                    Write-Host "Failed" -ForegroundColor Red
                }
                else
                {
                    Write-Host "Success" -ForegroundColor Green
                }

                Write-Host ""
            }
        }
    }
    catch
    {
        $grantedAllPermissions = $false;
        Write-Host "Failed to grant permissions '$($req.Scope)'. $($_) $newline" -ForegroundColor Red
    }

    if (!$grantedAllPermissions)
    {
        Write-Host "Could not grant all required permissions" -ForegroundColor DarkYellow
		
		
        try{
			$tenantAdminUrl = "";
			$hostUrl=([System.URI]$siteUrl).host

			if ($hostUrl.ToUpperInvariant().Contains("-ADMIN."))
			{
				$tenantAdminUrl = "https://$hostUrl";
			}
			else
			{
				# $temp=$hostUrl -ireplace [regex]::Escape("https://"), ""
				$tenantName = $hostUrl.substring(0,$hostUrl.indexof("."))
				$tenantAdminUrl = $hostUrl -ireplace [regex]::Escape("$tenantName"), "$tenantName-admin"
				$tenantAdminUrl = "https://$tenantAdminUrl"
			}

			$apiPermissionsUrl = "$tenantAdminUrl/_layouts/15/online/AdminHome.aspx#/webApiPermissionManagement"

			$tenantServicePrincipal = Get-PnPTenantServicePrincipal
			$azurePortalUrl = "https://portal.azure.com/#view/Microsoft_AAD_RegisteredApps/ApplicationMenuBlade/~/CallAnAPI/appId/$($tenantServicePrincipal.AppId)/isMSAApp~/false"

			Write-Host "The following pages have been opened in your browser. Please grant and approve the required permissions." -ForegroundColor DarkYellow
			Write-Host $apiPermissionsUrl -ForegroundColor DarkYellow
			Write-Host $azurePortalUrl -ForegroundColor DarkYellow
			Write-Host ""

			Start-Process $apiPermissionsUrl
			Start-Process $azurePortalUrl
		}
		catch
		{
			Write-Host "Failed to open webApiPermissionManagement page on browser" -ForegroundColor Red
		}
    }
}

Function CreateWebPage([string] $siteURL)
{
    try
    {
        if ([String]::IsNullOrWhiteSpace($siteUrl))
        {
            $siteurl = GetStringValueFromUser -message 'Please the enter site URL where you want to add the web part page'
        }

        $pageTitle= $webPartPageName.Substring(0, $webPartPageName.LastIndexOf('.'))

        Connect-PnPOnline -Url $siteURL -Interactive

        Get-PnPPage $pageTitle -ErrorAction SilentlyContinue -ErrorVariable getPageError -OutVariable getPageResult | Out-Null

        if (![String]::IsNullOrWhiteSpace($getPageResult))
        {
            $decision = Confirm -title "Confirm" -question "The web part page already exist. Do you want to delete and re-add it?";

            if ($decision -eq 0)
            {
                Remove-PnPPage -Identity "$pageTitle" -ErrorAction SilentlyContinue -ErrorVariable removePageError -OutVariable removePageResult | Out-Null

                if (![String]::IsNullOrWhiteSpace($removePageError))
                {
                    Write-Host "Failed to delete page. $removePageError" -ForegroundColor Red
                    return;
                }
            }
            else
            {
                Write-Host "Skipping web part page creation because it already exists." -ForegroundColor DarkYellow
                return;
            }
        }

        Write-Host "";
        Write-Host -ForegroundColor Cyan 'Adding web part page to site pages library';

        Add-PnPPage -Name $pageTitle -LayoutType SingleWebPartAppPage

        $page = Get-PnPPage -Identity $webPartPageName
        Add-PnPPageSection -Page $page -SectionTemplate OneColumn -Order 1

        Add-PnPPageWebPart -Page $pageTitle -Component $muhimbiWebpartName -Order 1
        $page.Publish()

        Write-Host -ForegroundColor Green 'Operation completed successfully.';

        return $true;
    }
    catch [System.Exception]
    {
        Write-Host -f Red "Failed to create web part page: "$_.Exception.Message
        return $false;
    }
}

Function CreateWebPageAllSite([string] $tenantURL)
{
	try
    {
		if ([String]::IsNullOrWhiteSpace($tenantURL))
        {
            $tenantURL = GetStringValueFromUser -message 'SharePoint tenant URL'
        }
		#$tenantURL = GetStringValueFromUser -message "SharePoint tenant URL"
		# Connect to SharePoint Online
		$CurrentConnection = Connect-PnPOnline -Url $tenantURL -Interactive

		# Get a list of all sites in the SharePoint tenant
		$sites = Get-PnPTenantSite -Connection $CurrentConnection
		
		# Filter out the sites that contain '/search' or '-my'
		$filteredSites = $sites | Where-Object { 
			$_.Url -notmatch '/search' -and $_.Url -notmatch '-my' 
		}

		$pageTitle= $webPartPageName.Substring(0, $webPartPageName.LastIndexOf('.'))
		foreach ($site in $filteredSites) {
			try
			{
				$siteUrl = $site.Url
				Write-Host "Adding webpart to: $siteUrl"
				Connect-PnPOnline -Url $SiteURL -Interactive
				
				Get-PnPPage $pageTitle -ErrorAction SilentlyContinue -ErrorVariable getPageError -OutVariable getPageResult | Out-Null

				if (![String]::IsNullOrWhiteSpace($getPageResult))
				{
					Write-Host "Skipping web part page creation because it already exists on $SiteURL." -ForegroundColor DarkYellow
				}
				else
				{
					try
					{
						Write-Host "";
						Write-Host -ForegroundColor Cyan 'Adding web part page to site pages library';

						Add-PnPPage -Name $pageTitle -LayoutType SingleWebPartAppPage

						$page = Get-PnPPage -Identity $webPartPageName
						Add-PnPPageSection -Page $page -SectionTemplate OneColumn -Order 1

						Add-PnPPageWebPart -Page $pageTitle -Component $muhimbiWebpartName -Order 1
						$page.Publish()

						Write-Host -ForegroundColor Green 'Webpart page added successfully.';
					}
					catch [System.Exception]
					{
						Write-Host -f Red "Failed to create web part page: "$_.Exception.Message
					}
				}
			}
			catch [System.Exception]
			{
				Write-Host -f Red "Failed to connect to site: "$_.Exception.Message
			}
		}
	}
	catch [System.Exception]
    {
        Write-Host -f Red "Error ocurred while adding webpart page to all sites: "$_.Exception.Message
        return $false;
    }	
}

Function RemoveWebPage()
{
    try
    {
        $siteurl = GetStringValueFromUser -message 'Please enter site URL from where to remove the web part page "MuhimbiFeatures.aspx"'
        $pageTitle= $webPartPageName.Substring(0, $webPartPageName.LastIndexOf('.'))

        Connect-PnPOnline -Url $siteURL -Interactive

        Remove-PnPPage -Identity $pageTitle -Recycle

        Write-Host "Web part page successfully deleted" -ForegroundColor Green

        return $true;
    }
    catch
    {
        Write-Host -f Red "Failed to delete web part page: "$_.Exception.Message
        return $false;
    }
}

Function RemoveWebPageFromAllSites()
{
    try
    {
		$tenantURL = GetStringValueFromUser -message "SharePoint tenant URL"
		# Connect to SharePoint Online
		$CurrentConnection = Connect-PnPOnline -Url $tenantURL -Interactive

		# Get a list of all sites in the SharePoint tenant
		$sites = Get-PnPTenantSite -Connection $CurrentConnection
		# Filter out the sites that contain '/search' or '-my'
		$filteredSites = $sites | Where-Object { 
			$_.Url -notmatch '/search' -and $_.Url -notmatch '-my' 
		}

		$pageTitle= $webPartPageName.Substring(0, $webPartPageName.LastIndexOf('.'))
		foreach ($site in $filteredSites) {
			try
			{
				$siteUrl = $site.Url
				Write-Host "Removing webpart page from: $siteUrl"
				Connect-PnPOnline -Url $siteUrl -Interactive
				
				Remove-PnPPage -Identity $pageTitle -Recycle -Force

				Write-Host "Web part page successfully deleted" -ForegroundColor Green
			}
			catch [System.Exception]
			{
				Write-Host -f Red "Failed to remove webpart page from site: "$_.Exception.Message
			}
		}
	}
	catch [System.Exception]
    {
        Write-Host -f Red "Error ocurred while removing webpart page from all sites: "$_.Exception.Message
        return $false;
    }
}

Function UninstallApp()
{
    try
    {
        $selection = "0";
        Write-Host "";

        do
        {
            Write-Host "From where do you want to remove the app?";
            Write-Host "1: Press '1' to remove from the tenant app catalog."
            Write-Host "2: Press '2' to remove from the site app catalog."
            $selection = Read-Host "Please make a selection"
        }
        until ($selection -eq '1' -Or $selection -eq '2')

        if ($selection -eq '1')
        {
            $scope = "tenant";
        }
        else
        {
            $scope = "site";
        }

        $url = GetStringValueFromUser -message "SharePoint $scope URL"
        Connect-PnPOnline -Url $url -Interactive

        # NOTE: Must run "Uninstall-PnPApp" BEFORE "Remove-PnPApp"
        if ($scope -eq 'tenant')
        {
            # $decision = Confirm -title "Confirm" -question "Do you want to uninstall the app from all sites in the tenant?";

            # if ($decision -eq '0')
            # {
            #     Uninstall-PnPApp -Identity $pspdfkitSolutionID -ErrorAction SilentlyContinue -ErrorVariable error
            # }
        }
        else
        {
            $decision = Confirm -title "Confirm" -question "Do you want to uninstall the app from this site?";

            if ($decision -eq '0')
            {
                Uninstall-PnPApp -Identity $muhimbiSolutionID -Scope Site -ErrorAction SilentlyContinue -ErrorVariable error
            }
        }

        $apps = $null;
        if ($scope -eq 'tenant')
        {
            $apps = Get-PnpApp
        }
        else
        {
            $apps = Get-PnPApp -Scope Site
        }

        if ($null -eq $apps)
        {
            Write-Host "There are no Muhimbi PDF converter online app deployed in the app catalog" -ForegroundColor DarkYellow
        }
        else
        {
            foreach ($app in $apps)
            {
                if ($app.Title -eq $muhimbiPackageName)
                {
                    if ($scope -eq 'tenant')
                    {
                        Remove-PnPApp -Identity $app.Id
                    }
                    else
                    {
                        Remove-PnPApp -Identity $app.Id -Scope Site
                    }

                    Write-Host "Muhimbi PDF converter online app successfully removed from the $scope app catalog" -ForegroundColor Green
                    break;
                }
            }
        }

        Write-Host "Operation completed" -ForegroundColor Green
    }
    catch
    {
        Write-Host -f Red "Failed to uninstall Muhimbi PDF Converter online app: "$_.Exception.Message
    }
}

Function GetStringValueFromUser([string] $message)
{
    $value = "";

    do
    {
        try
        {
            $value = Read-Host $message
            Write-Host '';
        }
        catch { }
    } until (![string]::IsNullOrWhitespace($value))

    return $value;
}

Function Confirm([string] $title, [string] $question, [int] $defaultChoice = 0)
{
    $choices  = '&Yes', '&No';
    return $Host.UI.PromptForChoice($title, $question, $choices, $defaultChoice);
}

Function TurnOnExternalSharing()
{
    try
    {
        $siteUrl = GetStringValueFromUser -message "SharePoint tenant URL"
        Connect-PnPOnline -Url $siteUrl -Interactive

        Set-PnPTenant -ShowAllUsersClaim $true

        Write-Host "External user sharing enabled" -ForegroundColor Green;
        Write-Host "Please Grant 'Read' permissions to 'All Users(membership)' group on tenant App Catalog or site app catalog manually"

    }
    catch
    {
        Write-Host "Failed to enable external user sharing." -ForegroundColor Red
        Write-Host $_ -ForegroundColor Red
    }
}

Function GiveFullControlToSiteCollection()
{
    try
    {
        $siteUrl = GetStringValueFromUser -message "Sharepoint site url in which you want to elevate the Muhimbi PDF Converter Online app."
        
		# App ID based on environment
        $appId = "388be915-e494-4691-8f25-f22a8d00a8c8"

        Connect-PnPOnline -Url $siteUrl -Interactive

        #First create a Read or Write permission entry for the app to the site. Currently unable to elevate
        $grant = Grant-PnPAzureADAppSitePermission -Permissions "Write" -Site $siteUrl -AppId $appId -DisplayName "SitesResourceSpecific"

        #Change the newly created Read/Write app site permission entry to FullControl
        Set-PnPAzureADAppSitePermission -Site $siteUrl -PermissionId $(($grant).Id) -Permissions "FullControl"
        
        
        Write-Host "App elevated on " $siteUrl -ForegroundColor Green;
		
		$decision = Confirm -title "Confirm" -question "Please confirm if you want to enable custom script (Please Note, Muhimbi realtime watermarking feature require custom script to be enabled)?";

		if ($decision -eq 0)
		{
			EnableCustomScript
		}
		else
		{
			Write-Host "Custom Script enable operation cancelled" -ForegroundColor DarkYellow
		}
		
    }
    catch
    {
		Write-Host "Failed to elevate the app on " $siteUrl -ForegroundColor Red
        Write-Host $_ -ForegroundColor Red
    }
}
Function GiveFullControlToAllSites()
{
    try
    {
		# App ID based on environment
        $appId = "388be915-e494-4691-8f25-f22a8d00a8c8"
		
		$tenantURL = GetStringValueFromUser -message "SharePoint tenant URL"
		# Connect to SharePoint Online
		$CurrentConnection = Connect-PnPOnline -Url $tenantURL -Interactive

		# Get a list of all sites in the SharePoint tenant
		$sites = Get-PnPTenantSite -Connection $CurrentConnection
		# Filter out the sites that contain '/search' or '-my'
		$filteredSites = $sites | Where-Object { 
			$_.Url -notmatch '/search' -and $_.Url -notmatch '-my' 
		}
		foreach ($site in $filteredSites) {
			try
			{
				$siteUrl = $site.Url
				Write-Host "Elevationg app on: $siteUrl"
				Connect-PnPOnline -Url $siteUrl -Interactive
				#First create a Read or Write permission entry for the app to the site. Currently unable to Set as FullControl
				$grant = Grant-PnPAzureADAppSitePermission -Permissions "Write" -Site $siteUrl -AppId $appId -DisplayName "SitesResourceSpecific"

				#Change the newly created Read/Write app site permission entry to FullControl
				Set-PnPAzureADAppSitePermission -Site $siteUrl -PermissionId $(($grant).Id) -Permissions "FullControl"
				
				
				Write-Host "App elevated on " $siteUrl -ForegroundColor Green;
			}
			catch [System.Exception]
			{
				Write-Host -f Red "Failed to elevate app on $siteUrl : "$_.Exception.Message
			}
		}
	}
	catch
    {
		Write-Host "Failed to connect to tenant site" -ForegroundColor Red
        Write-Host $_ -ForegroundColor Red
    }
}
Function RevokeFullControlFromSiteCollection() {
    try {
        $siteUrl = GetStringValueFromUser -message "SharePoint site URL from which you want to revoke elevation for the Muhimbi PDF Converter Online app"
        
        # App ID based on environment
        $appId = "388be915-e494-4691-8f25-f22a8d00a8c8"

        Connect-PnPOnline -Url $siteUrl -Interactive

        # Get the existing app site permission entry
        $permissions = Get-PnPAzureADAppSitePermission -Site $siteUrl -AppIdentity $appId
        
        if ($permissions) {
            # Revoke the app site permission entry
            foreach ($permission in $permissions) {
                Revoke-PnPAzureADAppSitePermission -Site $siteUrl -PermissionId $permission.Id -Force
                Write-Host "Elevation revoked from " $siteUrl -ForegroundColor Green
            }
        } else {
            Write-Host "No permissions found for app $appId on site $siteUrl" -ForegroundColor Yellow
        }
    } catch {
        Write-Host "Failed to revoke elevation from " $siteUrl -ForegroundColor Red
        Write-Host $_.Exception.Message -ForegroundColor Red
    }
}
Function RevokeFullControlFromAllSites()
{
	try
    {
		# App ID based on environment
        $appId = "388be915-e494-4691-8f25-f22a8d00a8c8"
		
		$tenantURL = GetStringValueFromUser -message "SharePoint tenant URL"
		# Connect to SharePoint Online
		$CurrentConnection = Connect-PnPOnline -Url $tenantURL -Interactive

		# Get a list of all sites in the SharePoint tenant
		$sites = Get-PnPTenantSite -Connection $CurrentConnection
		# Filter out the sites that contain '/search' or '-my'
		$filteredSites = $sites | Where-Object { 
			$_.Url -notmatch '/search' -and $_.Url -notmatch '-my' 
		}
		foreach ($site in $filteredSites) {
			try
			{
				$siteUrl = $site.Url
				Write-Host "Revoking elevation from: $siteUrl"
				Connect-PnPOnline -Url $siteUrl -Interactive
				# Get the existing app site permission entry
				$permissions = Get-PnPAzureADAppSitePermission -Site $siteUrl -AppIdentity $appId
				try
				{
					if ($permissions) {
						# Revoke the app site permission entry
						foreach ($permission in $permissions) {
							Revoke-PnPAzureADAppSitePermission -Site $siteUrl -PermissionId $permission.Id -Force
							Write-Host "Elevation revoked from " $siteUrl -ForegroundColor Green
						}
					} else {
						Write-Host "No permissions found for app $appId on site $siteUrl" -ForegroundColor Yellow
					}
				}
				catch
				{
					Write-Host "Failed to revoke permission from $siteUrl" -ForegroundColor Red
					Write-Host $_.Exception.Message -ForegroundColor Red
				}
			}
			catch{
				Write-Host "Failed to connect to $siteUrl or get all permissions " -ForegroundColor Red
				Write-Host $_.Exception.Message -ForegroundColor Red
			}
		}
	}
	catch
	{
		Write-Host "Failed to connect to tenant site or get all sites " -ForegroundColor Red
        Write-Host $_.Exception.Message -ForegroundColor Red
	}
}
Function EnableCustomScript()
{
	try
	{
		$adminSiteUrl = GetStringValueFromUser -message "Please Enter tenant admin site url"
		Connect-SPOService -Url $adminSiteUrl 
		
		$siteUrl = GetStringValueFromUser -message "Please Enter site url where you want to enable custom script"
		Set-SPOSite -Identity $siteUrl -DenyAddAndCustomizePages 0
		
		Write-Host "Custom script enabled on " $siteUrl -ForegroundColor Green;
		
	}
	catch
	{
		Write-Host "Failed to enable custom script on " $siteUrl -ForegroundColor Red
        Write-Host $_ -ForegroundColor Red
	}
}
Function EnableCustomScriptOnAllSites() {
    try {
        $adminSiteUrl = GetStringValueFromUser -message "Please Enter tenant admin site URL"
        Connect-SPOService -Url $adminSiteUrl 
        
        # Get all site collections
        $siteCollections = Get-SPOSite -Limit All | Where-Object { $_.Url -notmatch "/search" -and $_.Url -notmatch "-my" }

        foreach ($site in $siteCollections) {
            try {
                Set-SPOSite -Identity $site.Url -DenyAddAndCustomizePages 0
                Write-Host "Custom script enabled on " $site.Url -ForegroundColor Green
            } catch {
                Write-Host "Failed to enable custom script on " $site.Url -ForegroundColor Red
                Write-Host $_.Exception.Message -ForegroundColor Red
            }
        }
    } catch {
        Write-Host "Failed to retrieve site collections or connect to admin site" -ForegroundColor Red
        Write-Host $_.Exception.Message -ForegroundColor Red
    }
}

Function UninstallOldMuhimbiAddInAllSites
{
    $decision = Confirm -title "Confirm" -question "We are about to remove the old Muhimbi PDF Converter for SharePoint Online add-in from all site collections in your tenant including subsites, please confirm?";
    if ($decision -eq 0)
    {
        $tenantURL = GetStringValueFromUser -message "SharePoint tenant URL"
        # Connect to SharePoint Online
        $CurrentConnection = Connect-PnPOnline -Url $tenantURL -Interactive

        # Get a list of all sites in the SharePoint tenant
        $sites = Get-PnPTenantSite -Connection $CurrentConnection
		$filteredSites = $sites | Where-Object { 
			$_.Url -notmatch '/search' -and $_.Url -notmatch '-my' 
		}

        # Specify the add-in ID to uninstall
        $AppName = "Muhimbi PDF Converter for SharePoint Online"
        foreach ($site in $filteredSites) {
            $siteUrl = $site.Url
            Write-Host "Checking site: $siteUrl"
            Connect-PnPOnline -Url $SiteURL -Interactive
            try
            {
                $App = Get-PnPApp | Where {$_.Title -eq "Muhimbi PDF Converter for SharePoint Online"}
                $App
                if ($App) {
                    # Uninstall the add-in from the site
	                $App.Id
                    Write-Host "Uninstalling add-in from site: $siteUrl"
                    #Uninstall-PnPApp -Identity $App.Id
                } else {
                    Write-Host "Add-in is not installed on site: $siteUrl"
                }

                $SubSites = Get-PnPSubWeb -Recurse
                ForEach ($web in $SubSites)
                {
                    Write-Host $Web.URL
		            Connect-PnPOnline -Url $Web.URL -Interactive
		            try
		            {
			            $App = Get-PnPApp | Where {$_.Title -eq "Muhimbi PDF Converter for SharePoint Online"}
			            if ($App) {
        		            # Uninstall the add-in from the site
			                $App.Id
        		            Write-Host "Uninstalling add-in from site: $Web.URL"
        		            #Uninstall-PnPApp -Identity $App.Id
    			        } 
                        else 
                        {
        			        Write-Host "Add-in is not installed on site: $Web.URL"
    			        }
		            }
		            catch{}
                }

            }
            catch { }
        }
    }
    else
    {
        Write-Host "Uninstall action cancelled"
    }

}


try
{
    [System.Net.ServicePointManager]::SecurityProtocol = [System.Net.SecurityProtocolType]::Tls12;
    [Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Tls12
    # Unregister-PSRepository -Name PSGallery
    # Register-PSRepository -Default
    # Set-PSRepository -Name "PSGallery" -InstallationPolicy Trusted

    # PnP.PowerShell 2.1.1 does not work:
    # https://learn.microsoft.com/en-us/answers/questions/1196279/import-module-could-not-load-file-or-assembly-syst
    if ($null -ne (Get-InstalledModule -Name "PnP.PowerShell" -RequiredVersion 2.1.1 -ErrorAction SilentlyContinue)) {
        Uninstall-Module -Name "PnP.PowerShell" -RequiredVersion 2.1.1 -Force
    }

    if ($null -eq (Get-InstalledModule -Name "PnP.PowerShell" -RequiredVersion 1.12.0)) {
        Install-Module -Name "PnP.PowerShell" -RequiredVersion 1.12.0 -Force -AllowClobber
    }

    # Import-Module -FullyQualifiedName @{ModuleName = 'PnP.PowerShell'; ModuleVersion = '1.12.0' }
    Import-Module PnP.PowerShell
}
catch
{
    Write-Host "Failed to install dependencies. $newline" -ForegroundColor Red
    Write-Host $_ -ForegroundColor Red
    exit
}

Run
